shader{

	code= "
		uniform sampler2D		heightmapTexture;
		uniform vec3			sunLightVector;

		varying vec3			viewVector;

		float maxHeight = 0.1f;
		const float particlesDensity= 0.002;

		float sampleDensity(vec3 samplingPoint){
			float height= texture2D(heightmapTexture, samplingPoint.xz).x * maxHeight;

			if(-samplingPoint.y <= height)
			return 1;
				else
			return 0;
				
			float falloff= 1.0- clamp(-samplingPoint.y- height, 0.0, 0.2)*5.0;
			return particlesDensity*falloff;
		}

		void main(){
		
			
			const float maxDistance = 100.0f;
			const float bias = 0.00001f;
			const int numLinearSteps = 30;
			const int numBinarySteps = 2;
			
			viewVector = normalize(viewVector);

			float cosine = -viewVector.y;
			float hipotenuse = maxHeight / cosine;

			vec3 samplingVector = viewVector * hipotenuse;
			vec3 samplingVectorStep = -samplingVector / numLinearSteps;
			vec3 samplingPoint = vec3(gl_TexCoord[0].s, 0, gl_TexCoord[0].t) + samplingVector;

			float opticalDepth = 0.0;

			//Sample along the view vector.
			for(int i = 0; i < numLinearSteps; i++)
			{
				//Same code used for the view vector sampling (see above).
				vec3 samplingPoint2= samplingPoint;
				float cosine2= -sunLightVector.y;
				float samplingVectorLength2= samplingPoint2.y/cosine2/numLinearSteps;
				vec3 samplingVector2= -sunLightVector*samplingVectorLength2;
				float opticalDepth2= 0.0;
				
				//Sample along the sun light vector.
				for(int j= 0; j<= numLinearSteps; ++j){
					opticalDepth2+= sampleDensity(samplingPoint2) * samplingVectorLength2;
					samplingPoint2+= samplingVector2;
				}

				opticalDepth += sampleDensity(samplingPoint) * (hipotenuse / numLinearSteps);
				samplingPoint += samplingVectorStep;
			}


			//gl_FragColor= texture2D(heightmapTexture, samplingPoint.xz);
			gl_FragColor= 1.0 - opticalDepth;
			gl_FragColor.xyz = pow(gl_FragColor.xyz*1.5, 2.2);
		}
	"

	semantics{
		sunLightVector= SUN_LIGHT_DIRECTION
	}
}