shader
{
	code= "
	uniform float	fogDensity;
	uniform float	fogHeightFalloff;

	float getFogDensity(vec3 position, vec3 viewPosition)
	{
		//This fog effect is based on the effect used by the CryEngine2 and described
		//in the presentation 'Real-Time Atmospheric Effects in Games Revisited'.

		float depth = length(viewPosition - position);
		vec3 viewVector = position - viewPosition;

		//Calculate the fog density along the view ray.
		float densityAtViewerAltitude = exp(-viewPosition.y * fogHeightFalloff);
		float integral = densityAtViewerAltitude * depth;

		//To avoid artifacts caused by precision and singularity errors perform the
		//following operations only if the view vector has some vertical inclination.
		if(abs(normalize(viewVector).y) > 0.01)
		{

			float t = fogHeightFalloff * viewVector.y;
			integral = densityAtViewerAltitude * depth * (1.0 - exp(-t)) / t;
		}

		float density = 1.0 - exp(-fogDensity * integral);

		return density;
	}


	vec4 getFog(vec3 position, vec3 viewPosition)
	{
		vec3 color = vec3(2.0);
		float density = getFogDensity(position, viewPosition);

		//Return the fog color and opacity.
		return vec4(color, density);
	}
	"
	
	semantics
	{
		fogDensity = FOG_DENSITY
		fogHeightFalloff = FOG_HEIGHT_FALLOFF
	}
}