shader
{
	code= "
		#extension GL_EXT_geometry_shader4 : enable

		#define PARTICLE_WIDTH	0.005
		#define PARTICLE_HEIGHT	0.5

		uniform vec3			cameraPosition;
		uniform float			rainIntensity;

		varying in vec3			motionPath;
		varying in float		textureOffset;
		varying in float		scale;
		varying in float		intensityThreashold;

		varying out vec3		normal;
		varying out vec3		tangent;
		varying out vec3		binormal;

		void main()
		{

			if(intensityThreashold > rainIntensity)
				return;

			/*
			//Early out if this rain drop is outside the field of view.
			 vec4 cliPosition = gl_ModelViewProjectionMatrix * vec4(gl_PositionIn[0].xyz, 1);
			 cliPosition = cliPosition / cliPosition.w;
			 if(cliPosition.x < -1 || cliPosition.x > 1 || cliPosition.y < -1 || cliPosition.y > 1 || cliPosition.z < -1 || cliPosition.z > 1)
				return;
			*/
			
			//Calculate the view and side vectors.
			vec3 direction = normalize(-motionPath);
			vec3 viewVector = normalize(cameraPosition - gl_PositionIn[0].xyz);
			vec3 sideVector = normalize(cross(viewVector, direction));

			//Build the tangent space for the billboard.
			normal = - cross(direction, sideVector);
			tangent = -	sideVector;
			binormal = direction;

			float width = PARTICLE_WIDTH * scale;
			float height = PARTICLE_HEIGHT * scale;

			//Expand the original vertex into an axis aligned billboard formed by a triangle strip.

			//First point.
			gl_Position = gl_ModelViewProjectionMatrix * vec4(gl_PositionIn[0].xyz - sideVector * width, 1);
			gl_TexCoord[0] = vec4(textureOffset, 1, 0, 0);
			gl_TexCoord[1] = gl_Position;
			EmitVertex();

			//Second point.
			gl_Position = gl_ModelViewProjectionMatrix * vec4(gl_PositionIn[0].xyz - sideVector * width + direction * height, 1);
			gl_TexCoord[0] = vec4(textureOffset, 0, 0, 0);
			gl_TexCoord[1] = gl_Position;
			EmitVertex();

			//Third point.
			gl_Position = gl_ModelViewProjectionMatrix * vec4(gl_PositionIn[0].xyz + sideVector * width, 1);
			gl_TexCoord[0] = vec4(textureOffset + 1.0 / 16.0, 1, 0, 0);
			gl_TexCoord[1] = gl_Position;
			EmitVertex();

			//Fourth point.
			gl_Position = gl_ModelViewProjectionMatrix * vec4(gl_PositionIn[0].xyz + sideVector * width + direction * height, 1);
			gl_TexCoord[0] = vec4(textureOffset + 1.0 / 16.0, 0, 0, 0);
			gl_TexCoord[1] = gl_Position;
			EmitVertex();
		}
	"

	semantics
	{
		cameraPosition = CAMERA_POSITION
	}
}