shader
{
	code= "
		uniform vec3			cameraPosition;
		uniform float			elapsedTime;
	
		attribute vec3			startPosition;					//Start position of the motion path.
		attribute vec4			motionPathAndTextureOffset;		//Motion path.
		attribute vec4			attributes;						//Per particle attributes (X = textureOffset, Y = motionPhase, Z = scale, W = intensityThreashold).		

		varying out vec3		motionPath;
		varying out float		textureOffset;
		varying out float		scale;
		varying out float		intensityThreashold;

		void main()
		{
			//Pass the motion path and texture offset to the geometry shader.
			motionPath = motionPathAndTextureOffset.xyz;
			textureOffset = attributes.x;
			scale = attributes.z;
			intensityThreashold = attributes.w;

			//Calculate the position of the rain drop along its motion path. The rain drop loops the motion path
			//over and over, to do that we take the fractional part of the elapsed time to get a [0, 1] parametrization
			//that we use to place the particle on the path. An additional phase parameter is provided per rain drop to
			//provide randomness to the position of the drops since without it the drops would start and finish their motion
			//paths at the same time.
			float phase = attributes.y;
			vec3 position = startPosition + cameraPosition + motionPath * fract(phase + elapsedTime);

			gl_Position= vec4(position, 1.0);
		}
	"

	semantics
	{
		cameraPosition = CAMERA_POSITION
		startPosition = VERTEX_POSITION
		motionPathAndTextureOffset = VERTEX_TEXCOORD
		attributes = VERTEX_COLOR
	}
}