shader{

	code= "		
	#define MAX_OCCLUSION_QUADS		64
	#define ANGLE_BIAS				0.2
	
	uniform vec3					occlusionQuads[MAX_OCCLUSION_QUADS*4];
	uniform int						numOcclusionQuads;
					
	bool isSkyAccessibleToRay(vec3 rayOrigin, vec3 rayDirection){
		
		for(int quad= 0; quad< numOcclusionQuads; ++quad){
		
			//Get quad data.
			vec3 center= occlusionQuads[quad*4];
			vec3 vector1= occlusionQuads[quad*4+ 1];
			vec3 vector2= occlusionQuads[quad*4+ 2];
			vec2 dimensions= occlusionQuads[quad*4+ 3].xy;
			float halfLength1= dimensions.x;
			float halfLength2= dimensions.y;
			vec3 rectangleNormal= normalize(cross(vector1, vector2));

			//Ignore planes that are too sideways to the ray. These planes are ignored
			//because they're not likely to be hit by the ray and because they cause artifacts
			//on the sky lighting when eventually they are hit.
			if(abs(dot(rayDirection, rectangleNormal))< ANGLE_BIAS)
				continue;
			
			//Calculate intersection point.
			float t= dot(rayOrigin- center, rectangleNormal)/dot(rayDirection, -rectangleNormal);
			
			if(t> 0.0){
				vec3 intersectionPoint= rayOrigin+ rayDirection*t;
				
				//Check if intersection point is inside rectangle.
				vec3 aux= intersectionPoint- center;
				float u= abs(dot(aux, vector1));
				float v= abs(dot(aux, vector2));
				
				if(u<= halfLength1 && v<= halfLength2)
					return false;
			}
		}
		
		return true;
	}
	"
	
	semantics{
		occlusionQuads= SKY_LIGHTING_OCCLUSION_QUADS
		numOcclusionQuads= SKY_LIGHTING_NUM_OCCLUSION_QUADS
	}
}