shader{

	code= "
	
	//Spherical Harmonics coefficients.
	#define C1			0.282095
	#define C2			0.488603
	#define C3			1.092548
	#define C4			0.315392
	#define C5			0.546274
	
	uniform sampler3D	volumeTexture0;
	uniform sampler3D	volumeTexture1;
	uniform sampler3D	volumeTexture2;
	uniform sampler3D	volumeTexture3;
	uniform sampler3D	volumeTexture4;
	uniform sampler3D	volumeTexture5;
	uniform sampler3D	volumeTexture6;
	uniform float		volumeWidth;
	uniform float		volumeHeight;
	uniform float		volumeLength;
	uniform float		volumeTexelSize;

	vec3 sampleSkyLightingVolume(vec3 position, vec3 normal){
		
		//Calculate the sample position, offset it by half texel in the direction of the normal and convert it to texture coordinates.
		vec3 texCoord= (position.xzy + normal.xzy*volumeTexelSize*0.5)/vec3(volumeWidth, volumeLength, volumeHeight)+ vec3(0.5, 0.5, 0.0);

		//Discard pixels outside the volume.
		//if(texCoord.x< 0.0 || texCoord.x> 1.0 || texCoord.y< 0.0 || texCoord.y> 1.0 || texCoord.z< 0.0 || texCoord.z> 1.0)
			//discard;
		
		//Read the data from the volume textures.	
		vec4 t0= texture3D(volumeTexture0, texCoord);
		vec4 t1= texture3D(volumeTexture1, texCoord);
		vec4 t2= texture3D(volumeTexture2, texCoord);
		vec4 t3= texture3D(volumeTexture3, texCoord);
		vec4 t4= texture3D(volumeTexture4, texCoord);
		vec4 t5= texture3D(volumeTexture5, texCoord);
		vec4 t6= texture3D(volumeTexture6, texCoord);
		
		//Extract the coefficients.
		vec3 L00, L1m1, L10, L11, L2m2, L2m1, L20, L21, L22;
		L00= vec3(t0.x, t0.y, t0.z);
		L1m1= vec3(t0.w, t1.x, t1.y);
		L10= vec3(t1.z, t1.w, t2.x);
		L11= vec3(t2.y, t2.z, t2.w);
		L2m2= vec3(t3.x, t3.y, t3.z);
		L2m1= vec3(t3.w, t4.x, t4.y);
		L20= vec3(t4.z, t4.w, t5.x);
		L21= vec3(t5.y, t5.z, t5.w);
		L22= vec3(t6.x, t6.y, t6.z);
		
		//Sample the color from the Spherical Harmonics function.
		float aux0= normal.x*normal.x- normal.y*normal.y;
		float aux1= normal.z*normal.z;
		float aux2= normal.x*normal.y;
		float aux3= normal.x*normal.z;
		float aux4= normal.y*normal.z;
		float aux5= normal.x;
		float aux6= normal.y;
		float aux7= normal.z;
		
		vec3 lighting= 0.429043*L22*aux0*C5+
		0.743125*L20*aux1*C4+ 0.886227*L00*C1- 0.247708*L20*C4+
		2.0*0.429043*(L2m2*aux2+ L21*aux3+ L2m1*aux4)*C3+
		2.0*0.511664*(L11*aux5+ L1m1*aux6+ L10*aux7)*C2;

		return max(lighting, 0.0);
	}
	"
	
	semantics{
		volumeTexture0= SKY_LIGHTING_VOLUME_TEXTURE0
		volumeTexture1= SKY_LIGHTING_VOLUME_TEXTURE1
		volumeTexture2= SKY_LIGHTING_VOLUME_TEXTURE2
		volumeTexture3= SKY_LIGHTING_VOLUME_TEXTURE3
		volumeTexture4= SKY_LIGHTING_VOLUME_TEXTURE4
		volumeTexture5= SKY_LIGHTING_VOLUME_TEXTURE5
		volumeTexture6= SKY_LIGHTING_VOLUME_TEXTURE6
		volumeWidth= SKY_LIGHTING_VOLUME_WIDTH
		volumeHeight= SKY_LIGHTING_VOLUME_HEIGHT
		volumeLength= SKY_LIGHTING_VOLUME_LENGTH
		volumeTexelSize= SKY_LIGHTING_VOLUME_WORLDSPACE_TEXEL_SIZE
	}
}