shader{

	code= "
	uniform sampler2D	colorTexture;
	uniform sampler2D	normalTexture;
	uniform sampler2D	specularTexture;
	uniform float		alphaTestTreshold;
	uniform float		specularPower;
	
	varying mat3		tangentSpaceMatrix;
	varying vec3		geometryNormal;
	varying float		linearDepth;
	varying vec3		position;
	
	void main(){

		vec4 color= texture2D(colorTexture, gl_TexCoord[0].st);

		//Discard pixel if alpha is bellow treshold.
		if(color.a<= alphaTestTreshold)
			discard;
	
		//Color and glossiness.
		float glossiness= texture2D(specularTexture, gl_TexCoord[0].st).x;
		gl_FragData[0]= vec4(color.rgb, glossiness);
	
		//World-space normal.
		vec3 normal= texture2D(normalTexture, gl_TexCoord[0].st).xyz*2.0- 1.0;		
		gl_FragData[1]= vec4(normalize(normal*transpose(tangentSpaceMatrix)), specularPower);
		
		//World-space geometry normal.
		gl_FragData[2]= vec4(normalize(geometryNormal), 1.0);
		
		//Linear depth.
		gl_FragData[3]= vec4(linearDepth, 1.0, 1.0, 1.0);
		
		//Position.
		gl_FragData[4]= vec4(position, 0.0);

		//Motion vector.
		gl_FragData[5]= vec4(calculateMotionVector(), 0, 0);
	}"

	includes{
		file = resources/shaders/engine/deferredRendering/gBuffer/motionBlur.fs
	}

	semantics{
		colorTexture= MATERIAL_COLOR_TEXTURE
		normalTexture= MATERIAL_NORMAL_TEXTURE
		specularTexture= MATERIAL_SPECULAR_TEXTURE
		alphaTestTreshold= MATERIAL_ALPHA_TEST_TRESHOLD
		specularPower= MATERIAL_SPECULAR_POWER
	}
}