shader
{
	code= "
	uniform vec3	cameraPosition;
	uniform mat4	objectMatrix;
	uniform mat4	previousMatrix;
	
	attribute vec3	normal;
	attribute vec3	tangent;
	attribute vec3	binormal;
	attribute float	layerWeight;

	varying mat3	tangentSpaceMatrix;
	varying vec3	geometryNormal;
	varying float	linearDepth;
	varying vec3	position;
	varying vec3	tangentSpaceViewVector;
	varying float	terrainLayerWeight;

	varying vec4	previousProjection;
	varying vec4	currentProjection;
	
	void main()
	{
		gl_TexCoord[0] = gl_MultiTexCoord0;
		gl_Position = ftransform();

		mat3 objectMatrixNoTranslation = mat3(objectMatrix);

		//Calculate the inverse transpose of the object matrix that is used to compensate for the eventual object's non-uniform scaling when transforming vectors.
		mat3 inverseTransposeObjectMatrix = transpose(inverse(objectMatrixNoTranslation));
		
		//Calculate the world-space tangent matrix.
		tangentSpaceMatrix = inverseTransposeObjectMatrix * mat3(tangent, binormal, normal);		
		
		//Calculate information for the G-Buffer.
		linearDepth = -(gl_ModelViewMatrix * gl_Vertex).z;
		position = (objectMatrix * gl_Vertex).xyz;
		geometryNormal = inverseTransposeObjectMatrix * normal;
		tangentSpaceViewVector = (cameraPosition - position) * tangentSpaceMatrix;

		//Calculate and store the vertex's projection for the current and previous frame.
		currentProjection = gl_Position;
		previousProjection = previousMatrix * gl_Vertex;

		//Terrain layer blending =================================================
		#ifdef USE_TERRAIN_LAYER_BLENDING

		terrainLayerWeight = layerWeight;

		#endif
		//========================================================================
	}
	"
	
	semantics
	{
		objectMatrix = OBJECT_TRANSFORM_MATRIX
		previousMatrix = OBJECT_PREVIOUS_TRANSFORM_MODELVIEW_PROJECTION_MATRIX
		normal = VERTEX_NORMAL
		tangent = VERTEX_TANGENT
		binormal = VERTEX_BINORMAL
		layerWeight = TERRAIN_MATERIAL_WEIGHT
		cameraPosition = CAMERA_POSITION
	}
}