shader{

	code= "
	uniform float		elapsedTime;

	varying vec4		previousProjection;
	varying vec4		currentProjection;

	vec2 calculateMotionVector()
	{
		vec2 previousScreenSpacePosition = previousProjection.xy / previousProjection.w;
		vec2 currentScreenSpacePosition = currentProjection.xy / currentProjection.w;
		vec2 motion = (currentScreenSpacePosition - previousScreenSpacePosition) * 0.5;

		//Fix artifact that occurs when the point being processed was behind the near plane in the previous frame.
		if(abs(previousProjection.z) / previousProjection.w < 0)
			motion = -motion;

		//Scale the vector by blur amount and elapsed time.
		const float blurScale = 3.0;
		motion *=  blurScale * elapsedTime;

		//Clamp the vector length to avoid excessive blur.
		const float maxBlurLength= 0.04;
		if(length(motion) > maxBlurLength)
			motion = normalize(motion) * maxBlurLength;

		return motion;
	}
	"

	semantics{
		elapsedTime = ELAPSED_TIME
	}
}