shader{

	code= "
	#define PI			3.141592

	float cookTorrance(vec3 normal, vec3 lightVector, vec3 viewVector, float specularMultiplier, float roughness)
	{
		vec3 N = normal;
		vec3 L = lightVector;
		vec3 V = -viewVector;
		vec3 H = normalize(L + V);

		float HdotN = dot(H, N);
		float LdotN = max(dot(L, N), 0.0);
		float VdotN = dot(V, N);
		float VdotH = dot(V, H);

		//We use the Gaussian distribution instead of the Beckam distribution because it gives a more predictable behavior of the specular shape.
		float angle = acos(HdotN);
		float D = exp(-(angle*angle)/(2*roughness*roughness))/(roughness*sqrt(2.0*PI));

		float F = specularMultiplier;						//The specular multiplier contains the fresnel term and the glossiness.

		//Geometric term.
		float G1 = (2.0 * HdotN * VdotN) / (VdotH + 0.0001);
		float G2 = (2.0 * HdotN * LdotN) / (VdotH + 0.0001);
		float G1G2 = max(min(G1, G2), 0.0);					//The minimum between G1 and G2 is clamped to zero to avoid artifacts.
		float G = min(1.0, G1G2);

		float R = (D * F * G) / (VdotN + 0.0001);

		return R;
	}

	vec3 calculateLighting(vec3 color, float specularRoughness, float specularMultiplier, vec3 normal, vec3 lightVector, vec3 viewVector, vec3 lightColor)
	{
		float diffuse = max(dot(normal, lightVector), 0.0);
		float specular = cookTorrance(normal, lightVector, -viewVector, specularMultiplier, specularRoughness) * 10;

		return (color * diffuse + specular) * lightColor;
	}
	"
}