shader{

	code= "

	uniform vec3		lightPosition;
	uniform vec4		lightColor;
	uniform float		lightRadius;
	uniform float		lightIntensity;

	vec3 getPointLighting(vec3 color, vec3 position, vec3 normal, vec3 cameraPosition, float specularMultiplier, float specularRoughness){
		
		//Calculate the light and view vectors.
		vec3 lightVector = normalize(lightPosition - position);
		vec3 viewVector = normalize(cameraPosition - position);

		//Calculate the lighting.
		vec3 lighting = calculateLighting(color, specularRoughness, specularMultiplier, normal, lightVector, viewVector, lightColor);

		//Calculate the lighting attenuation.
		float attenuation= pow(max(1- length(lightPosition - position) / lightRadius, 0.0), 2.0);

		return lighting * attenuation * lightIntensity;
	}
	"

	includes{
		file= resources/shaders/engine/deferredRendering/lightingModel.sh
	}

	semantics{
		lightPosition = LIGHT_POSITION
		lightColor = LIGHT_COLOR
		lightRadius = LIGHT_RADIUS
		lightIntensity = LIGHT_INTENSITY
	}
}