shader{

	code= "
	uniform sampler2D	colorTexture;
	uniform sampler2D	positionTexture;
	uniform sampler2D	normalTexture;
	uniform sampler2D	lightMaskTexture;
	uniform vec3		cameraPosition;
	
	void main()
	{	
		vec3 color= texture2D(colorTexture, gl_TexCoord[0].st).rgb;
		vec3 position= texture2D(positionTexture, gl_TexCoord[0].st).xyz;
		vec3 normal= texture2D(normalTexture, gl_TexCoord[0].st).xyz;

		//Calculate the view vector.
		vec3 viewVector = normalize(cameraPosition - position);
		
		float specularMultiplier = 0.0;
		float specularRoughness = 0.0;
		vec3 lighting= getSunLighting(color, normal, specularMultiplier, specularRoughness, viewVector);

		float shadow= texture2D(lightMaskTexture, gl_TexCoord[0].st).x;

		gl_FragColor= vec4(lighting*shadow, 1.0);
	}
	"

	includes{
		file= resources/shaders/engine/deferredRendering/sunLighting.sh
	}

	semantics{
		colorTexture= GBUFFER_COLOR_TEXTURE
		positionTexture= GBUFFER_POSITION_TEXTURE
		normalTexture= GBUFFER_NORMAL_TEXTURE
		lightMaskTexture= RAYTRACING_LIGHT_MASK_TEXTURE
		cameraPosition= CAMERA_POSITION
	}
}