shader{

	code= "
	#define EYE_RODS_SENSITIVITY	0.05
	#define EYE_CONES_SENSITIVITY	0.1
	#define MIN_EYE_LUMINANCE		0.7
	#define MAX_EYE_LUMINANCE		1.0
	
	uniform sampler2D				sceneAverageLuminanceTexture;
	uniform sampler2D				previousLuminanceTexture;
	uniform float					elapsedTime;
		
	void main(){
	
		//Get the averaged scene luminance by reading the last mipmap level from the texture (assuming 512x512 texture dimensions).
		float sceneAverageLuminance= texture2DLod(sceneAverageLuminanceTexture, vec2(0.0, 0.0), 9).r;
		float currentLuminance= exp(sceneAverageLuminance);
		
		//Read the previous scene luminance.
		float previousLuminance= texture2D(previousLuminanceTexture, vec2(0.0, 0.0)).r;
		
		//Perform eye adaptation.
		float tau= 0.04/(0.04+ currentLuminance);
		float rate= tau*EYE_RODS_SENSITIVITY+ (1.0- tau)*EYE_CONES_SENSITIVITY;
		float finalLuminance= previousLuminance + (currentLuminance - previousLuminance)*(1.0- exp(-elapsedTime*rate));
		
		//Don't let the eyes adapt to extremely high or low luminance values.
		finalLuminance= clamp(finalLuminance, MIN_EYE_LUMINANCE, MAX_EYE_LUMINANCE);
		
		gl_FragColor= vec4(finalLuminance, 0.0, 0.0, 0.0);
	}
	"
	
	semantics{
		sceneAverageLuminanceTexture= HDR_SCENE_AVERAGE_LUMINANCE_TEXTURE
		previousLuminanceTexture= HDR_PREVIOUS_FRAME_LUMINANCE_TEXTURE
		elapsedTime= ELAPSED_TIME
	}
}