shader{

	code= "
	#define MIDDLE_GREY				0.18
	#define COMPRESSION_WHITE		1.0
	
	uniform sampler2D				texture;
	
	varying float					sceneLuminance;

	void main()
	{
		const mat3 YxyMatrix= mat3(	0.4124, 0.3576, 0.1805,
									0.2126, 0.7152, 0.0722,
									0.0193, 0.1192, 0.9505);
									
		const mat3 rgbMatrix= mat3(	3.2405, -1.5371, -0.4985,
									-0.9693, 1.8760, 0.0416,
									0.0566, -0.2040, 1.0572);
	
		//Get the pixel color.
		vec3 color= texture2D(texture, gl_TexCoord[0].st).rgb;
											
		//Convert pixel color from RGB to CIE Yxy.
		vec3 aux= color*YxyMatrix;
		vec3 Yxy= vec3(aux.y, aux.x/(aux.x + aux.y + aux.z), aux.y/(aux.x + aux.y + aux.z));					
										
		//Calculate the new luminance.	
		Yxy.x= Yxy.x*MIDDLE_GREY/sceneLuminance;
		
		//Compress.
		Yxy.x= Yxy.x*(1.0+ Yxy.x/(COMPRESSION_WHITE*COMPRESSION_WHITE))/(1.0+ Yxy.x);
		
		//Convert pixel color back to RGB.
		color= vec3(Yxy.y*(Yxy.x/Yxy.z), Yxy.x, (1.0- Yxy.y- Yxy.z)*(Yxy.x/Yxy.z))*rgbMatrix;

		//Apply gamma correction to the pixel color.
		color = pow(color, 1.0/2.2);

		//Stylish color.
		/*
		color = mix(Yxy.x, color, 0.2);
		color *= vec3(1.2, 1.2, 0.925);
		color = pow(color, 1.0);
		*/
				
		gl_FragColor= vec4(color, 1.0);
	}
	"
}