shader{

	code= "	
	uniform sampler2D	propagationTexture0;
	uniform sampler2D	propagationTexture1;
	uniform sampler2D	propagationTexture2;
	uniform sampler2D	propagationTexture3;
	uniform float		propagationVolumeWidthResolution;
	uniform float		propagationVolumeHeightResolution;
	uniform float		propagationVolumeLengthResolution;
	

	SH readSphericalHarmonic(vec2 texCoord){
		SH result;
		result.L00= texture2D(propagationTexture0, texCoord).xyz;
		result.L1m1= texture2D(propagationTexture1, texCoord).xyz;
		result.L10= texture2D(propagationTexture2, texCoord).xyz;
		result.L11= texture2D(propagationTexture3, texCoord).xyz;
		
		return result;
	}
	
	SH getIrradianceFromAdjacentCell(vec3 cellDirection){
		
		//Calculate the texture coordinates to address the unwrapped texture.
		vec2 offset= vec2((cellDirection.x/propagationVolumeWidthResolution+ cellDirection.z)/propagationVolumeHeightResolution, cellDirection.y/propagationVolumeLengthResolution);
		vec2 texCoord= gl_TexCoord[0].xy+ offset;
		
		//Clamp the x coordinate of the texture coordinate to avoid sampling outside this layer.
		float layer= floor(texCoord.x*propagationVolumeHeightResolution);
		texCoord.x= clamp(texCoord.x, 1.0/propagationVolumeHeightResolution*layer, 1.0/propagationVolumeHeightResolution*(layer+ 1.0));
		
		SH adjacentCellSH= readSphericalHarmonic(texCoord);		
		vec3 irradianceColor= sampleColorFromSH(adjacentCellSH, -cellDirection.xzy);
		return createLightLobeSH(irradianceColor, -cellDirection.xzy);
	}
	
	void main(){
		
		SH centerCell;
		centerCell= addSH(centerCell, getIrradianceFromAdjacentCell(vec3(-1, 0, 0)));
		centerCell= addSH(centerCell, getIrradianceFromAdjacentCell(vec3(1, 0, 0)));
		centerCell= addSH(centerCell, getIrradianceFromAdjacentCell(vec3(0, -1, 0)));
		centerCell= addSH(centerCell, getIrradianceFromAdjacentCell(vec3(0, 1, 0)));
		centerCell= addSH(centerCell, getIrradianceFromAdjacentCell(vec3(0, 0, -1)));
		centerCell= addSH(centerCell, getIrradianceFromAdjacentCell(vec3(0, 0, 1)));
		
		gl_FragData[0]= vec4(centerCell.L00, 0.0);
		gl_FragData[1]= vec4(centerCell.L1m1, 0.0);
		gl_FragData[2]= vec4(centerCell.L10, 0.0);
		gl_FragData[3]= vec4(centerCell.L11, 0.0);
	}
	"
	
	includes{
		file= resources/shaders/engine/lightPropagationVolumes/sphericalHarmonics.sh
	}
	
	semantics{
		propagationTexture0= LIGHT_PROPAGATION_VOLUME_TEXTURE0
		propagationTexture1= LIGHT_PROPAGATION_VOLUME_TEXTURE1
		propagationTexture2= LIGHT_PROPAGATION_VOLUME_TEXTURE2
		propagationTexture3= LIGHT_PROPAGATION_VOLUME_TEXTURE3
		propagationVolumeWidthResolution= LIGHT_PROPAGATION_VOLUME_WIDTH_RESOLUTION
		propagationVolumeHeightResolution= LIGHT_PROPAGATION_VOLUME_HEIGHT_RESOLUTION
		propagationVolumeLengthResolution= LIGHT_PROPAGATION_VOLUME_LENGTH_RESOLUTION
	}
}