shader{

	code= "
	uniform sampler2D		depthTexture;	
	uniform vec3			cameraPosition;
	uniform vec3			frustumUpperLeftVector;
	uniform vec3			frustumUpperRightVector;
	uniform vec3			frustumLowerLeftVector;
	uniform vec3			frustumLowerRightVector;
	
	
	//Gets the pixel's depth.
	float getPixelDepth(vec2 texcoord){
		return texture2D(depthTexture, texcoord).x;
	}
	
	//Reconstructs the position by fetching the pixel's depth and calculating the interpolated frustum vector.
	vec3 reconstructPositionFromDepth(vec2 texcoord){
		float depth= getPixelDepth(texcoord);

		vec3 aux1 = mix(frustumUpperLeftVector, frustumUpperRightVector, texcoord.x);
		vec3 aux2 = mix(frustumLowerLeftVector, frustumLowerRightVector, texcoord.x);
		vec3 frustumVector = mix(aux2, aux1, texcoord.y);

		return cameraPosition+ frustumVector*depth;		
	}
	
	//Reconstructs the position from the pixel's texture coordinates and the interpolated frustum vector.
	vec3 reconstructPositionFromDepth(vec2 texcoord, vec3 frustumVector){
		float depth= getPixelDepth(texcoord);
		return cameraPosition+ frustumVector*depth;		
	}
	
	//Reconstructs the position from the pixel's depth and the interpolated frustum vector.
	vec3 reconstructPositionFromDepth(float depth, vec3 frustumVector){
		return cameraPosition+ frustumVector*depth;		
	}
	"
	
	semantics{
		depthTexture= GBUFFER_DEPTH_TEXTURE
		cameraPosition= CAMERA_POSITION
		frustumUpperLeftVector= POSITION_RECONSTRUCTION_UPPER_LEFT_VECTOR
		frustumUpperRightVector= POSITION_RECONSTRUCTION_UPPER_RIGHT_VECTOR
		frustumLowerLeftVector= POSITION_RECONSTRUCTION_LOWER_LEFT_VECTOR
		frustumLowerRightVector= POSITION_RECONSTRUCTION_LOWER_RIGHT_VECTOR
	}
}