shader{

	code= "

		uniform sampler2D	colorTexture;
		uniform sampler2D	raysDirectionsTexture;
		uniform vec2		offsets[8];

		void main(){

			//Apply a box blur filter that will reduce the aliasing of the raytracing effect and
			//correct the borders artifacts caused by the resolution mismatch between the effect
			//and the main rendering. This box filter samples only pixels that are covered by the
			//rendering mask, all others are ignored. 
				
			vec3 result = texture2D(colorTexture, gl_TexCoord[0].st).rgb;
			int numSourcePixels= 1;

			//For each neighbour...
			for(int i= 0; i< 8; ++i){

				//Is the neighbour covered by the mask?
				if(texture2D(raysDirectionsTexture, gl_TexCoord[0].st + offsets[i]).a > 0.0){

					result+= texture2D(colorTexture, gl_TexCoord[0].st + offsets[i]).rgb;
					++ numSourcePixels;
				}
			}

			//The result is the average color from all the covered neighbours.
			gl_FragColor= vec4(result/numSourcePixels, 1.0);
		}
	"

	semantics{
		colorTexture= RAYTRACING_CURRENT_EFFECT_TEXTURE
		raysDirectionsTexture= RAYTRACING_RAYS_DIRECTIONS_TEXTURE
	}
}