shader
{
	code= "
	in vec2			texCoord;
	in float		circleOfConfusionMultiplier;
	in float		focalPointDepth;
	out vec4		gl_FragColor;

	void main()
	{
		const float MAX_FOG_RADIUS = 0.03;
		const float DOF_BLUR_AMOUNT = 0.03;
		const float MAX_DOF_BLUR = 0.02;
		const float MIN_DEPTH_DIFFERENCE = 1.0;

		//Get the fog density for this pixel and convert it into a blur radius.
		vec3 position = reconstructPositionFromDepth(texCoord);
		float fogDensity = getFogDensity(position, cameraPosition);
		float fogCircleOfConfusion = fogDensity * fogDensity * MAX_FOG_RADIUS;		//The fog density is squared to reduce the blur near the viewer.

		//Calculate the difference between the depth of the current pixel and the depth of the focal point. 
		float depth = texture2D(depthTexture, texCoord).x;
		float distanceToFocalPoint = abs(depth - focalPointDepth);

		//The distance to the focal point must be greater than the threashold before the image starts to get blurred.
		distanceToFocalPoint = max(distanceToFocalPoint - MIN_DEPTH_DIFFERENCE, 0.0);

		//Increase the depth of field if we're looking far away.
		float blurAmount = DOF_BLUR_AMOUNT / focalPointDepth;

		//Calculate the amount of blurring based on the distance to the focal point.
		float dofCircleOfConfusion = distanceToFocalPoint * blurAmount;
		
		//Clamp the blur amount to the maximum allowed.
		dofCircleOfConfusion = min(dofCircleOfConfusion, MAX_DOF_BLUR);

		//Multiply by the circle of confusion multiplier generated by the eye focal adaptation.
		dofCircleOfConfusion *= circleOfConfusionMultiplier;

		gl_FragColor = vec4(fogCircleOfConfusion + dofCircleOfConfusion);
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/version.sh
		file = resources/shaders/engine/positionFromDepth.sh
		file = resources/shaders/engine/atmosphere/fog/fog.sh
	}
}