shader{

	code= "
	#define	NUM_SAMPLES		30
	#define SKIP_THREASHOLD	0.001

	uniform sampler2D	motionVectorsTexture;
	uniform sampler2D	colorTexture;

	void main()
	{
		vec2 motionVector = texture2D(motionVectorsTexture, gl_TexCoord[0].st).xy;

		vec2 texCoordStep = -motionVector / (NUM_SAMPLES - 1);
		vec2 texCoord = gl_TexCoord[0].st + texCoordStep;
		vec3 result = 0.0;
		float totalWeight = 0.0;
		
		//Sample along the opposite direction of the motion vector
		//weighting each sample by its distance to the source pixel.
		for(int i = 0; i < NUM_SAMPLES; i++, texCoord += texCoordStep)
		{
			float weight = (1 - (float) i / NUM_SAMPLES);
			result += texture2D(colorTexture, texCoord).rgb * weight;
			totalWeight += weight;
		}
		
		result /= totalWeight;

		gl_FragColor = vec4(result, 0.0);
	}
	"

	semantics
	{
		motionVectorsTexture = GBUFFER_MOTION_BLUR_VECTORS_TEXTURE
	}
}