shader{

	code= "
	
		#define MAX_SAMPLES			32
		
		uniform sampler2D			positionTexture;
		uniform sampler2D			normalTexture;
		uniform sampler2D			depthTexture;
		uniform sampler2D			jitterTexture;
		
		uniform vec3				tlVector;
		uniform vec3				trVector;
		uniform vec3				blVector;
		uniform vec3				brVector;
		
		uniform vec2				vectors[MAX_SAMPLES];				
		uniform int					numSamples;
		uniform float				ambientOcclusionDistance;
		uniform float				intensity;
		
		
		vec3 getSamplePosition(vec2 texcoord){	
		
			return texture2D(positionTexture, texcoord);
		}
		
		void main(){
		
			vec3 position= getSamplePosition(gl_TexCoord[0].st);
			vec3 normal= texture2D(normalTexture, gl_TexCoord[0].st);
			float depth= texture2D(depthTexture, gl_TexCoord[0].st).x;
			
			float jitterAngle= texture2D(jitterTexture, gl_TexCoord[2].st).x*2.0*3.14;
			mat2 jitterMatrix= mat2(cos(jitterAngle), sin(jitterAngle), -sin(jitterAngle), cos(jitterAngle));
			
			//Scale the sampling radius according to the distance.
			float radiusScale= 1.0/depth;

			float occlusion= 0.0;
			
			for(int i= 0; i< numSamples; ++i){
				
				vec3 samplePosition= getSamplePosition(gl_TexCoord[0].st+ vectors[i]*jitterMatrix*radiusScale);
				
				float angleAttenuation= max(dot(normalize(samplePosition- position), normal), 0.0);
				float distance= length(position- samplePosition);	
				float distanceAttenuation= max((ambientOcclusionDistance- distance)/ambientOcclusionDistance, 0.0);
										
				occlusion+= distanceAttenuation*angleAttenuation;
			}

			for(int i= 0; i< numSamples; ++i){
				
				vec3 samplePosition= getSamplePosition(gl_TexCoord[0].st+ vectors[i]*jitterMatrix*radiusScale*0.1);
				
				float angleAttenuation= max(dot(normalize(samplePosition- position), normal), 0.0);
				float distance= length(position- samplePosition);	
				float distanceAttenuation= max((ambientOcclusionDistance- distance)/ambientOcclusionDistance, 0.0);
										
				occlusion+= distanceAttenuation*angleAttenuation;
			}
							
			gl_FragColor= vec4(pow(1.0- occlusion/numSamples/2, intensity));
		}
	"
	
	semantics{
		positionTexture = GBUFFER_POSITION_TEXTURE
		normalTexture= GBUFFER_GEOMETRY_NORMAL_TEXTURE
		depthTexture= GBUFFER_DEPTH_TEXTURE
		jitterTexture= SHADOW_SAMPLING_NOISE_TEXTURE
		tlVector= FRUSTUM_TOP_LEFT_VECTOR
		trVector= FRUSTUM_TOP_RIGHT_VECTOR
		blVector= FRUSTUM_BOTTOM_LEFT_VECTOR
		brVector= FRUSTUM_BOTTOM_RIGHT_VECTOR
		vectors= SSAO_SAMPLING_VECTORS
		numSamples= SSAO_NUM_SAMPLES
		ambientOcclusionDistance= SSAO_DISTANCE
		intensity= SSAO_INTENSITY
	}
}