shader{

	code= "
		#define MAX_GAUSSIAN_COEFFICIENTS	32
		#define DISTANCE_TRESHOLD			0.5
		
		uniform sampler2D	colorTexture;
		uniform sampler2D	normalTexture;
		uniform sampler2D	depthTexture;
		
		uniform float		gaussian[MAX_GAUSSIAN_COEFFICIENTS];
		uniform int			numSamples;
		uniform float		ssaoBlurRadius;
		
		void main(){
			vec2 sampleStepSize= vec2(ssaoBlurRadius/(numSamples- 1), 0.0);
			vec2 samplingTexcoord= gl_TexCoord[0].st- sampleStepSize*(numSamples- 1)*0.5;
			
			float sum= 0.0;
			float occlusion= 0.0;
			float depth= texture2D(depthTexture, gl_TexCoord[0].st).x;
			
			//Blur.
			for(int i= 0; i< numSamples- 1; ++i){
			
				float sampleDepth= texture2D(depthTexture, samplingTexcoord).x;
				float distanceCoef= max(DISTANCE_TRESHOLD- abs(depth- sampleDepth), 0.0);
				
				occlusion+= texture2D(colorTexture, samplingTexcoord).x*gaussian[i]*distanceCoef;
				sum+= gaussian[i]*distanceCoef;
				samplingTexcoord+= sampleStepSize;
			}
			
			gl_FragColor= vec4(occlusion/sum);
		}
	"
	
	semantics{
		colorTexture= SSAO_TEXTURE
		normalTexture= GBUFFER_NORMAL_TEXTURE
		depthTexture= GBUFFER_DEPTH_TEXTURE
		gaussian= SSAO_GAUSSIAN_BLUR_COEFFICIENTS
		numSamples= SSAO_GAUSSIAN_BLUR_NUM_SAMPLES
		ssaoBlurRadius= SSAO_BLUR_RADIUS
	}

}