shader{

	code= "
	const int			NUM_SAMPLES = 9;
	const float			MAX_SAMPLING_DISTANCE = 0.1;
	
	uniform float		viewportWidth;
	uniform float		viewportHeight;
	uniform sampler2D	lightMaskTexture;
	uniform vec2		blurDirection;


	void main()
	{
		vec2 texelSize = vec2(1 / viewportWidth, 1 / viewportHeight);
		vec2 texCoord = gl_TexCoord[0].st - texelSize * blurDirection * (NUM_SAMPLES - 1) / 2;
		vec2 step = texelSize * blurDirection;

		float centerPixelDepth = getPixelDepth(gl_TexCoord[0].st);

		float result = 0;
		float acccumulatedWeight = 0;
		for(int sample = 0; sample < NUM_SAMPLES; sample++)
		{
			//Sample the light mask.
			float shadowSample = texture(lightMaskTexture, texCoord);

			//Calculate a distance based weighting.
			float sampleDepth = getPixelDepth(texCoord);
			float distanceWeight = max(1.0 - abs(centerPixelDepth - sampleDepth) / MAX_SAMPLING_DISTANCE, 0.0);

			//Accumulate the weighted sample and weight.
			result += shadowSample * distanceWeight;
			acccumulatedWeight += distanceWeight;

			//Proceed to the next sample.
			texCoord += step;
		}

		//Normalize the result.
		result /= acccumulatedWeight;

		gl_FragColor= vec4(result, result, result, 1.0);
	}
	"

	includes
	{
		file = resources/shaders/engine/deferredRendering/gBuffer/gBufferRead.sh
	}

	semantics
	{
		viewportWidth = VIEWPORT_WIDTH
		viewportHeight = VIEWPORT_HEIGHT
	}
}