shader{

	code= "
	#define CASCADE_DEPTH_BIAS0		0.0003
	#define CASCADE_DEPTH_BIAS1		0.0004
	#define CASCADE_DEPTH_BIAS2		0.0004
	#define CASCADE_DEPTH_BIAS3		0.0005
	#define CASCADE_DEPTH_BIAS4		0.001

	uniform sampler2D		shadowMap0;
	uniform sampler2D		shadowMap1;
	uniform sampler2D		shadowMap2;
	uniform sampler2D		shadowMap3;
	uniform sampler2D		shadowMap4;
	uniform mat4			shadowMapMatrix0;
	uniform mat4			shadowMapMatrix1;
	uniform mat4			shadowMapMatrix2;
	uniform mat4			shadowMapMatrix3;
	uniform mat4			shadowMapMatrix4;
	uniform float			shadowMapSplitDistance0;
	uniform float			shadowMapSplitDistance1;
	uniform float			shadowMapSplitDistance2;
	uniform float			shadowMapSplitDistance3;
	uniform float			shadowMapSplitDistance4;
	uniform float			shadowMapSunLightRadius0;
	uniform float			shadowMapSunLightRadius1;
	uniform float			shadowMapSunLightRadius2;
	uniform float			shadowMapSunLightRadius3;
	uniform float			shadowMapSunLightRadius4;
	
	
	float sampleCascadedShadowMap(vec3 position, float depth, vec2 screenSpacePosition){
	
		const mat4 offsetMatrix= mat4(	0.5, 0.0, 0.0, 0.5, 
										0.0, 0.5, 0.0, 0.5, 
										0.0, 0.0, 0.5, 0.5, 
										0.0, 0.0, 0.0, 1.0);
										
		if(depth<= shadowMapSplitDistance0)
			return samplePercentageCloserSoftShadows(shadowMap0, shadowMapMatrix0*vec4(position, 1.0)*offsetMatrix, screenSpacePosition, shadowMapSunLightRadius0, CASCADE_DEPTH_BIAS0);
		else if(depth<= shadowMapSplitDistance1)
			return samplePercentageCloserSoftShadows(shadowMap1, shadowMapMatrix1*vec4(position, 1.0)*offsetMatrix, screenSpacePosition, shadowMapSunLightRadius1, CASCADE_DEPTH_BIAS1);
		else if(depth<= shadowMapSplitDistance2)
			return samplePercentageCloserSoftShadows(shadowMap2, shadowMapMatrix2*vec4(position, 1.0)*offsetMatrix, screenSpacePosition, shadowMapSunLightRadius2, CASCADE_DEPTH_BIAS2);
		else if(depth<= shadowMapSplitDistance3)
			return samplePercentageCloserSoftShadows(shadowMap3, shadowMapMatrix3*vec4(position, 1.0)*offsetMatrix, screenSpacePosition, shadowMapSunLightRadius3, CASCADE_DEPTH_BIAS3);
		else if(depth<= shadowMapSplitDistance4)
			return samplePercentageCloserSoftShadows(shadowMap4, shadowMapMatrix4*vec4(position, 1.0)*offsetMatrix, screenSpacePosition, shadowMapSunLightRadius4, CASCADE_DEPTH_BIAS4);
		else
			return 1.0;
	}
	"
	
	includes{
		file= resources/shaders/engine/shadows/softShadowMapping.sh
	}
	
	semantics{
		shadowMap0= CASCADED_SHADOW_MAP0
		shadowMap1= CASCADED_SHADOW_MAP1
		shadowMap2= CASCADED_SHADOW_MAP2
		shadowMap3= CASCADED_SHADOW_MAP3
		shadowMap4= CASCADED_SHADOW_MAP4
		shadowMapMatrix0= CASCADED_SHADOW_MAP0_MATRIX
		shadowMapMatrix1= CASCADED_SHADOW_MAP1_MATRIX
		shadowMapMatrix2= CASCADED_SHADOW_MAP2_MATRIX
		shadowMapMatrix3= CASCADED_SHADOW_MAP3_MATRIX
		shadowMapMatrix4= CASCADED_SHADOW_MAP4_MATRIX
		shadowMapSplitDistance0= CASCADED_SHADOW_MAP0_SPLIT_DISTANCE
		shadowMapSplitDistance1= CASCADED_SHADOW_MAP1_SPLIT_DISTANCE
		shadowMapSplitDistance2= CASCADED_SHADOW_MAP2_SPLIT_DISTANCE
		shadowMapSplitDistance3= CASCADED_SHADOW_MAP3_SPLIT_DISTANCE
		shadowMapSplitDistance4= CASCADED_SHADOW_MAP4_SPLIT_DISTANCE
		shadowMapSunLightRadius0= CASCADED_SHADOW_MAP0_UV_SPACE_SUN_LIGHT_RADIUS
		shadowMapSunLightRadius1= CASCADED_SHADOW_MAP1_UV_SPACE_SUN_LIGHT_RADIUS
		shadowMapSunLightRadius2= CASCADED_SHADOW_MAP2_UV_SPACE_SUN_LIGHT_RADIUS
		shadowMapSunLightRadius3= CASCADED_SHADOW_MAP3_UV_SPACE_SUN_LIGHT_RADIUS
		shadowMapSunLightRadius4= CASCADED_SHADOW_MAP4_UV_SPACE_SUN_LIGHT_RADIUS
	}
}