shader{

	code= "
	uniform float		shadowSoftness;
	uniform float		shadowDepthBias;
	uniform sampler2D	shadowMapNX;
	uniform sampler2D	shadowMapPX;
	uniform sampler2D	shadowMapNY;
	uniform sampler2D	shadowMapPY;
	uniform sampler2D	shadowMapNZ;
	uniform sampler2D	shadowMapPZ;
	uniform mat4		shadowMapMatrixNX;
	uniform mat4		shadowMapMatrixPX;
	uniform mat4		shadowMapMatrixNY;
	uniform mat4		shadowMapMatrixPY;
	uniform mat4		shadowMapMatrixNZ;
	uniform mat4		shadowMapMatrixPZ;

	float sampleCubicShadowMap(vec3 pixelPosition, vec3 lightPosition, vec2 screenSpacePosition){

		const mat4 offsetMatrix= mat4(	0.5, 0.0, 0.0, 0.5, 
										0.0, 0.5, 0.0, 0.5, 
										0.0, 0.0, 0.5, 0.5, 
										0.0, 0.0, 0.0, 1.0);

		//Find which shadow map should we sample from by finding the gratest coordinate (X, Y and Z)
		//of the vector pointing from the light to the pixel.
		vec3 direction = normalize(pixelPosition - lightPosition);
		if(abs(direction.x) >= abs(direction.y) && abs(direction.x) >= abs(direction.z))
		{
			if(direction.x < 0)
			{
				vec4 shadowMapTexCoord = shadowMapMatrixNX * vec4(pixelPosition, 1) * offsetMatrix;
				return sampleSoftShadowMap(shadowMapNX, shadowMapTexCoord / shadowMapTexCoord.w, shadowSoftness, screenSpacePosition, shadowDepthBias);
			}
			else
			{
				vec4 shadowMapTexCoord = shadowMapMatrixPX * vec4(pixelPosition, 1) * offsetMatrix;
				return sampleSoftShadowMap(shadowMapPX, shadowMapTexCoord / shadowMapTexCoord.w, shadowSoftness, screenSpacePosition, shadowDepthBias);
			}
		}
		//Y.
		else if(abs(direction.y) >= abs(direction.x) && abs(direction.y) >= abs(direction.z))
		{
			if(direction.y < 0)
			{
				vec4 shadowMapTexCoord = shadowMapMatrixNY * vec4(pixelPosition, 1) * offsetMatrix;
				return sampleSoftShadowMap(shadowMapNY, shadowMapTexCoord / shadowMapTexCoord.w, shadowSoftness, screenSpacePosition, shadowDepthBias);
			}
			else
			{
				vec4 shadowMapTexCoord = shadowMapMatrixPY * vec4(pixelPosition, 1) * offsetMatrix;
				return sampleSoftShadowMap(shadowMapPY, shadowMapTexCoord / shadowMapTexCoord.w, shadowSoftness, screenSpacePosition, shadowDepthBias);
			}
		}
		//Z.
		else if(abs(direction.z) >= abs(direction.x) && abs(direction.z) >= abs(direction.y))
		{
			if(direction.z < 0)
			{
				vec4 shadowMapTexCoord = shadowMapMatrixNZ * vec4(pixelPosition, 1) * offsetMatrix;
				return sampleSoftShadowMap(shadowMapNZ, shadowMapTexCoord / shadowMapTexCoord.w, shadowSoftness, screenSpacePosition, shadowDepthBias);
			}
			else
			{
				vec4 shadowMapTexCoord = shadowMapMatrixPZ * vec4(pixelPosition, 1) * offsetMatrix;
				return sampleSoftShadowMap(shadowMapPZ, shadowMapTexCoord / shadowMapTexCoord.w, shadowSoftness, screenSpacePosition, shadowDepthBias);
			}
		}

		return 1.0;
	}
	"
	
	includes{
		file= resources/shaders/engine/shadows/softShadowMapping.sh
	}
	
	semantics{
		shadowSoftness = LIGHT_SHADOW_SOFTNESS
		shadowDepthBias = LIGHT_SHADOW_DEPTH_BIAS
		shadowMapNX = SHADOW_MAP_NX
		shadowMapPX = SHADOW_MAP_PX
		shadowMapNY = SHADOW_MAP_NY
		shadowMapPY = SHADOW_MAP_PY
		shadowMapNZ = SHADOW_MAP_NZ
		shadowMapPZ = SHADOW_MAP_PZ
		shadowMapMatrixNX = SHADOW_MAP_NX_MATRIX
		shadowMapMatrixPX = SHADOW_MAP_PX_MATRIX
		shadowMapMatrixNY = SHADOW_MAP_NY_MATRIX
		shadowMapMatrixPY = SHADOW_MAP_PY_MATRIX
		shadowMapMatrixNZ = SHADOW_MAP_NZ_MATRIX
		shadowMapMatrixPZ = SHADOW_MAP_PZ_MATRIX
	}
}