shader{

	code= "
	uniform sampler2D		shadowMap0;
	uniform sampler2D		shadowMap1;
	uniform sampler2D		shadowMap2;
	uniform sampler2D		shadowMap3;
	uniform mat4			shadowMapMatrix0;
	uniform mat4			shadowMapMatrix1;
	uniform mat4			shadowMapMatrix2;
	uniform float			shadowMapSunLightRadius0;
	uniform float			shadowMapSunLightRadius1;
	uniform float			shadowMapSunLightRadius2;


	bool isInsideShadowMap(vec4 texcoord)
	{
		return texcoord.x>= 0 && texcoord.x<=1 && texcoord.y>= 0 && texcoord.y<=1;
	}
	
	float sampleOverlappedShadowMap(vec3 position, vec3 cameraPosition, vec2 screenSpacePosition){
		
		const mat4 offsetMatrix= mat4(	0.5, 0.0, 0.0, 0.5, 
										0.0, 0.5, 0.0, 0.5, 
										0.0, 0.0, 0.5, 0.5, 
										0.0, 0.0, 0.0, 1.0);
		
		//Lets see if the pixel is covered by the first shadow map and if so then sample from it.
		vec4 shadowMapTexcoord0 = shadowMapMatrix0*vec4(position, 1.0)*offsetMatrix;

		if(isInsideShadowMap(shadowMapTexcoord0))
			return samplePercentageCloserSoftShadows(shadowMap0, shadowMapTexcoord0, screenSpacePosition, shadowMapSunLightRadius0);


		//Lets see if the pixel is covered by the second shadow map and if so then sample from it.
		vec4 shadowMapTexcoord1 = shadowMapMatrix1*vec4(position, 1.0)*offsetMatrix;
		
		if(isInsideShadowMap(shadowMapTexcoord1))
			return samplePercentageCloserSoftShadows(shadowMap1, shadowMapTexcoord1,  screenSpacePosition, shadowMapSunLightRadius1);
		

		//Lets see if the pixel is covered by the third shadow map and if so then sample from it.
		vec4 shadowMapTexcoord2 = shadowMapMatrix2*vec4(position, 1.0)*offsetMatrix;

		if(isInsideShadowMap(shadowMapTexcoord2))
			return samplePercentageCloserSoftShadows(shadowMap2, shadowMapTexcoord2,  screenSpacePosition, shadowMapSunLightRadius2);
		
		return 1.0;
	}
	"

	includes{
		file= resources/shaders/engine/shadows/softShadowMapping.sh
	}

	semantics{
		shadowMap0= OVERLAPPED_SHADOW_MAP0
		shadowMap1= OVERLAPPED_SHADOW_MAP1
		shadowMap2= OVERLAPPED_SHADOW_MAP2
		shadowMapMatrix0= OVERLAPPED_SHADOW_MAP0_MATRIX
		shadowMapMatrix1= OVERLAPPED_SHADOW_MAP1_MATRIX
		shadowMapMatrix2= OVERLAPPED_SHADOW_MAP2_MATRIX
		shadowMapSunLightRadius0= OVERLAPPED_SHADOW_MAP0_UV_SPACE_SUN_LIGHT_RADIUS
		shadowMapSunLightRadius1= OVERLAPPED_SHADOW_MAP1_UV_SPACE_SUN_LIGHT_RADIUS
		shadowMapSunLightRadius2= OVERLAPPED_SHADOW_MAP2_UV_SPACE_SUN_LIGHT_RADIUS
	}
}