shader
{
	code= "
	#extension GL_EXT_shader_image_load_store : enable

	uniform int									voxelVolumeResolution;

	uniform sampler3D							voxelizationTextureNX;
	uniform sampler3D							voxelizationTexturePX;
	uniform sampler3D							voxelizationTextureNY;
	uniform sampler3D							voxelizationTexturePY;
	uniform sampler3D							voxelizationTextureNZ;
	uniform sampler3D							voxelizationTexturePZ;

	layout (rgba16f) coherent uniform image3D	voxelTextureNX;
	layout (rgba16f) coherent uniform image3D	voxelTexturePX;
	layout (rgba16f) coherent uniform image3D	voxelTextureNY;
	layout (rgba16f) coherent uniform image3D	voxelTexturePY;
	layout (rgba16f) coherent uniform image3D	voxelTextureNZ;
	layout (rgba16f) coherent uniform image3D	voxelTexturePZ;

	vec4 getVoxelColor(vec3 textureCoordinates, sampler3D voxelizationTexture)
	{
		float aux = 1.0 / (voxelVolumeResolution * 4);

		//Read how many fragments were used to generate this voxel.
		float counter = texture(voxelizationTexture, textureCoordinates + vec3(0, 0, 3) * aux).x;

		//If the voxel contains fragments then return their averaged color if not then return an empty voxel.
		if(counter > 0.0)
		{
			//Read the sum of the color channels from the textures.
			float r = texture(voxelizationTexture, textureCoordinates + vec3(0, 0, 0) * aux).x;
			float g = texture(voxelizationTexture, textureCoordinates + vec3(0, 0, 1) * aux).x;
			float b = texture(voxelizationTexture, textureCoordinates + vec3(0, 0, 2) * aux).x;

			//Return a voxel with the color average and full opacity.
			vec3 colorAverage = vec3(r, g, b) / counter;
			return vec4(colorAverage, 1.0);
		}
		else
		{
			//Return an empty voxel.
			return vec4(0.0, 0.0, 0.0, 0.0);
		}
	}

	void main()
	{
		float texelSize = 1.0 / (voxelVolumeResolution * 4.0);

		for(unsigned int layer = 0; layer < voxelVolumeResolution; layer++)
		{
			vec3 voxelCoordinates = vec3(gl_FragCoord.xy, layer);
			vec3 textureCoordinates = voxelCoordinates / voxelVolumeResolution;

			//Read the anisotropic voxel colors.
			vec4 voxelColorNX = getVoxelColor(textureCoordinates, voxelizationTextureNX);
			vec4 voxelColorPX = getVoxelColor(textureCoordinates, voxelizationTexturePX);
			vec4 voxelColorNY = getVoxelColor(textureCoordinates, voxelizationTextureNY);
			vec4 voxelColorPY = getVoxelColor(textureCoordinates, voxelizationTexturePY);
			vec4 voxelColorNZ = getVoxelColor(textureCoordinates, voxelizationTextureNZ);
			vec4 voxelColorPZ = getVoxelColor(textureCoordinates, voxelizationTexturePZ);

			//Store the results into the destination textures.
			imageStore(voxelTextureNX, ivec3(voxelCoordinates), voxelColorNX);
			imageStore(voxelTexturePX, ivec3(voxelCoordinates), voxelColorPX);
			imageStore(voxelTextureNY, ivec3(voxelCoordinates), voxelColorNY);
			imageStore(voxelTexturePY, ivec3(voxelCoordinates), voxelColorPY);
			imageStore(voxelTextureNZ, ivec3(voxelCoordinates), voxelColorNZ);
			imageStore(voxelTexturePZ, ivec3(voxelCoordinates), voxelColorPZ);
		}
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/version.sh
	}

	semantics
	{
		voxelVolumeResolution = VOXEL_VOLUME_RESOLUTION
	}
}