shader
{
	code= "
	#extension GL_EXT_shader_image_load_store : enable

	uniform int									mipmapLevel;
	uniform int									mipmapResolution;

	uniform	sampler3D							voxelTextureNX;
	uniform	sampler3D							voxelTexturePX;
	uniform	sampler3D							voxelTextureNY;
	uniform	sampler3D							voxelTexturePY;
	uniform	sampler3D							voxelTextureNZ;
	uniform	sampler3D							voxelTexturePZ;

	layout (rgba16f) coherent uniform image3D	voxelTextureMipmapNX;
	layout (rgba16f) coherent uniform image3D	voxelTextureMipmapPX;
	layout (rgba16f) coherent uniform image3D	voxelTextureMipmapNY;
	layout (rgba16f) coherent uniform image3D	voxelTextureMipmapPY;
	layout (rgba16f) coherent uniform image3D	voxelTextureMipmapNZ;
	layout (rgba16f) coherent uniform image3D	voxelTextureMipmapPZ;


	vec4 getMipmappedVoxelColor(vec3 texCoord, sampler3D voxelTexture)
	{
		float lowerMipmapResolution = mipmapResolution * 2.0;
		float lowerMipmapTexelSize = 1.0 / lowerMipmapResolution;

		vec3 texelTexCoord = texCoord - vec3(1, 1, 1) * lowerMipmapTexelSize / 4.0;

		//Read the texels.
		vec4 t1 = textureLod(voxelTexture, texelTexCoord + vec3(0, 0, 0) * lowerMipmapTexelSize, mipmapLevel - 1);
		vec4 t2 = textureLod(voxelTexture, texelTexCoord + vec3(1, 0, 0) * lowerMipmapTexelSize, mipmapLevel - 1);
		vec4 t3 = textureLod(voxelTexture, texelTexCoord + vec3(1, 1, 0) * lowerMipmapTexelSize, mipmapLevel - 1);
		vec4 t4 = textureLod(voxelTexture, texelTexCoord + vec3(0, 1, 0) * lowerMipmapTexelSize, mipmapLevel - 1);
		vec4 t5 = textureLod(voxelTexture, texelTexCoord + vec3(0, 0, 1) * lowerMipmapTexelSize, mipmapLevel - 1);
		vec4 t6 = textureLod(voxelTexture, texelTexCoord + vec3(1, 0, 1) * lowerMipmapTexelSize, mipmapLevel - 1);
		vec4 t7 = textureLod(voxelTexture, texelTexCoord + vec3(1, 1, 1) * lowerMipmapTexelSize, mipmapLevel - 1);
		vec4 t8 = textureLod(voxelTexture, texelTexCoord + vec3(0, 1, 1) * lowerMipmapTexelSize, mipmapLevel - 1);

		float totalOpacity = t1.a + t2.a + t3.a + t4.a + t5.a + t6.a + t7.a + t8.a;
		
		//Calculate the weighted average of the voxel colors by summing of the voxel colors multiplied by their respective opacities
		//and dividing the result by the sum of the opacities. A small bias is used in the division to avoid dividing by zero.
		vec3 color = (t1.rgb * t1.a + t2.rgb * t2.a + t3.rgb * t3.a + t4.rgb * t4.a + t5.rgb * t5.a + t6.rgb * t6.a + t7.rgb * t7.a + t8.rgb * t8.a) / (totalOpacity + 0.00001);
		
		//Ther resulting alpha is simply the average of the voxels opacities.
		float alpha = totalOpacity / 8.0;

		return vec4(color, alpha);
	}

	void main()
	{
		for(unsigned int layer = 0; layer < mipmapResolution; layer++)
		{
			//Calculate the mipmap colors for the voxel.
			vec3 textureCoordinates = vec3(gl_FragCoord.xy, layer) / mipmapResolution;

			vec4 voxelColorNX = getMipmappedVoxelColor(textureCoordinates, voxelTextureNX);
			vec4 voxelColorPX = getMipmappedVoxelColor(textureCoordinates, voxelTexturePX);
			vec4 voxelColorNY = getMipmappedVoxelColor(textureCoordinates, voxelTextureNY);
			vec4 voxelColorPY = getMipmappedVoxelColor(textureCoordinates, voxelTexturePY);
			vec4 voxelColorNZ = getMipmappedVoxelColor(textureCoordinates, voxelTextureNZ);
			vec4 voxelColorPZ = getMipmappedVoxelColor(textureCoordinates, voxelTexturePZ);

			//Store the results onto the destination textures.
			vec3 voxelCoordinates = vec3(gl_FragCoord.xy - vec2(0.5), layer);

			imageStore(voxelTextureMipmapNX, ivec3(voxelCoordinates), voxelColorNX);
			imageStore(voxelTextureMipmapPX, ivec3(voxelCoordinates), voxelColorPX);
			imageStore(voxelTextureMipmapNY, ivec3(voxelCoordinates), voxelColorNY);
			imageStore(voxelTextureMipmapPY, ivec3(voxelCoordinates), voxelColorPY);
			imageStore(voxelTextureMipmapNZ, ivec3(voxelCoordinates), voxelColorNZ);
			imageStore(voxelTextureMipmapPZ, ivec3(voxelCoordinates), voxelColorPZ);
		}
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/version.sh
	}

	semantics
	{
		voxelVolumeResolution = VOXEL_VOLUME_RESOLUTION
	}
}