shader
{
	code= "
	#extension GL_EXT_shader_image_load_store : enable

	uniform int								voxelVolumeResolution;

	layout (r32f) coherent uniform image3D	voxelTextureNX;
	layout (r32f) coherent uniform image3D	voxelTexturePX;
	layout (r32f) coherent uniform image3D	voxelTextureNY;
	layout (r32f) coherent uniform image3D	voxelTexturePY;
	layout (r32f) coherent uniform image3D	voxelTextureNZ;
	layout (r32f) coherent uniform image3D	voxelTexturePZ;

	void main()
	{
		//Clear all the layers of the volume, the number of layers is VOLUME_RESOLUTION * 4
		//due to the layout of the RGB color components and the fragment counter on the texture.
		for(unsigned int layer = 0; layer < voxelVolumeResolution * 4; layer++)
		{
			vec3 voxelCoordinates = vec3(gl_FragCoord.xy, layer);
			imageStore(voxelTextureNX, ivec3(voxelCoordinates), vec4(0.0, 0.0, 0.0, 0.0));
			imageStore(voxelTexturePX, ivec3(voxelCoordinates), vec4(0.0, 0.0, 0.0, 0.0));
			imageStore(voxelTextureNY, ivec3(voxelCoordinates), vec4(0.0, 0.0, 0.0, 0.0));
			imageStore(voxelTexturePY, ivec3(voxelCoordinates), vec4(0.0, 0.0, 0.0, 0.0));
			imageStore(voxelTextureNZ, ivec3(voxelCoordinates), vec4(0.0, 0.0, 0.0, 0.0));
			imageStore(voxelTexturePZ, ivec3(voxelCoordinates), vec4(0.0, 0.0, 0.0, 0.0));
		}
	}
	"

	includes
	{
		file = resources/shaders/engine/voxelConeTracing/version.sh
	}

	semantics
	{
		voxelVolumeResolution = VOXEL_VOLUME_RESOLUTION
	}
}